/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Clippings extension for Mozilla Firefox.
 *
 * The Initial Developer of the Original Code is 
 * Alex Eng <ateng@users.sourceforge.net>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

//
// Integration with host application
//


var gClippingsExec = {
  initOnStartup: false,

  newFromClipboard: function () 
  {
    var rv = gClippings.newFromClipboard();
    if (! rv) {
      alert("Clipboard is empty or does not contain textual data.");
    }
  },

  newFromTextField: function ()
  {
    // Get the node where the context menu is invoked from.
    var node = document.popupNode;  // XPCNativeWrapper -> HTMLInputElement
                                    // or HTMLTextAreaElement
    if (node instanceof HTMLInputElement || node instanceof HTMLTextAreaElement) {
      gClippings.newClippingFromText(node.value);
    }
    else {
      gClippings.log("In gClippingsExec.newFromTextField: node = " + node);
    }
  },

  newFromSelection: function ()
  {
    var focusedWnd = document.commandDispatcher.focusedWindow;
    var selection = focusedWnd.getSelection();
    var selectionStr = selection.toString();

    if (selectionStr) {
      gClippings.newClippingFromText(selectionStr);	
    }
    else {
      alert("No text selected. Please select text first.");
    }
  },

  openClippingsManager: function () 
  {
    window.open('chrome://clippings/content/clippings.xul', 'clippings_wndobj',
		'chrome');
  },

  initClippingsPopup: function (aPopup) 
  {
    var popup = aPopup;
    var rootNode = gClippings.RDFSvc.GetResource("http://clippings.mozdev.org/rdf/user-clippings");

    try {
      // If Clippings data file doesn't exist, it will be created automatically.
      if (! gClippings.dataSrc) {
	gClippings.dataSrc = gClippings.RDFSvc.GetDataSourceBlocking(gClippings.getCurrentProfileURL() + "/" + CLIPDAT_FILE_NAME);
      }
    }
    catch (e) {
      gClippings.log(e);
    }

    popup.database.AddDataSource(gClippings.dataSrc);

    try {
      gClippings.RDFContainer = gClippings.RDFContainerUtils.MakeSeq(gClippings.dataSrc, rootNode);
      gClippings.RDFContainer = gClippings.RDFContainer.QueryInterface(Components.interfaces.nsIRDFContainer);
    }
    catch (e) {
      gClippings.log(e);
    }

    popup.builder.rebuild();
    this.initOnStartup = true;
  },

  insertClippingText: function (aText) 
  {
    gClippings.copyTextToClipboard(aText);

    // Paste clipping.  The following function is defined in
    // "chrome://global/content/globalOverlay.js"
    goDoCommand("cmd_paste");
    // SIDE EFFECT: The clipping text will remain on the system clipboard.
  }
};


// Drag 'n drop support
var clippingsStatusBarObserver = {
  onDragOver: function (event, flavour, session)
  {
    event.target.setAttribute("ae-dnd", "true");
  },

  onDragExit: function (event, session)
  {
    event.target.removeAttribute("ae-dnd");
  },

  onDrop: function (event, transferData, session)
  {
    if (! gClippingsExec.initOnStartup) {
      // The RDF data source has to be initialized if it has not already been
      // done so, otherwise RDF node creation will fail, and the new Clippings
      // entry will never be created.
      // This initialization is done in the code for the Clippings popup menu's
      // `onpopupshowing' event handler.
      gClippingsExec.initClippingsPopup(document.getElementById("ae-clippings-popup-1"));
    }

    var text = transferData.data;
    var newEntryURI = gClippings.newClippingFromText(text);
  },

  getSupportedFlavours: function ()
  {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/unicode");
    return flavours;
  }
};


window.addEventListener("load", ae_clippings_initContextMenu, true);


function ae_clippings_initContextMenu(event)
{
  var hostAppContextMenu = document.getElementById("contentAreaContextMenu");
  hostAppContextMenu.addEventListener("popupshowing", ae_clippings_initContextMenuItem, false);
  hostAppContextMenu.addEventListener("popuphiding", ae_clippings_hideContextMenuItem, false);
}


function ae_clippings_initContextMenuItem(event)
{
  var clippingsPopup1 = document.getElementById("ae-clippings-popup-1");
  var clippingsPopup2 = document.getElementById("ae-clippings-popup-2");

  // HACK!  Although this event handler is attached to the browser's context
  // menu, it gets called 2 times: for the browser context menu, and for the
  // Clippings submenu.  We only want the following to execute when the 
  // browser's context menu is being shown, not the Clippings submenu.
  if (event.target.id == "contentAreaContextMenu") {
    if (gContextMenu.onTextInput) {
      var addEntryCmd = document.getElementById("ae_new_clipping_from_text_field");
      var node = document.popupNode;
      // Do our own checking to ensure that the node is an HTML text field.
      if (node instanceof HTMLInputElement || node instanceof HTMLTextAreaElement) {
	addEntryCmd.setAttribute("disabled", node.value == "");
      }
      else {
	addEntryCmd.setAttribute("disabled", "true");
      }
      clippingsPopup1.hidden = false;
      clippingsPopup2.hidden = true;
    }
    else if (gContextMenu.isTextSelected) {
      clippingsPopup1.hidden = true;
      clippingsPopup2.hidden = false;
    }
  }
}


function ae_clippings_hideContextMenuItem(event)
{
  var clippingsPopup1 = document.getElementById("ae-clippings-popup-1");
  var clippingsPopup2 = document.getElementById("ae-clippings-popup-2");

  // HACK!  Although this event handler is attached to the browser's context
  // menu, it gets called 2 times: for the browser context menu, and for the
  // Clippings submenu.  We only want the following to execute when the
  // browser's context menu is being hidden, not the Clippings submenu.
  if (event.target.id == "contentAreaContextMenu") {
    clippingsPopup1.hidden = true;
    clippingsPopup2.hidden = true;
  }
}


